<?php if(!defined('_VALID')){header('Status: 404 Not Found');die;}
## FS class
# @author legolas558
#
# centralized filesystem operations

class	FS {

var $_batch_mode;
var $_critical_mode;

	function FS($failback = false, $batch = false) {
		// atomical batch sequences not yet supported
		$this->_batch_mode = $batch;
		// choose to fail (with server error) or to simply report back a false return value
		$this->_critical_mode = !$failback;
	}
	
	function _failure($msg) {
		global $d_root, $my;
		include $d_root.'includes/servererror.php';
//		ob_end_clean();
		ob_start();
		$explanation = '<hr />';
		if ($my->gid==5)
			$explanation.='If you cannot solve the error by its message, check that the correct write permissions are set on the target path';
		else
			$explanation.='Contact the system administrator and complain about this error';
		service_msg('Filesystem operation failed', $msg, $explanation, 'stop');
		exit;
	}
	
	function _fix_path($path) {
		return substr($path, strlen($GLOBALS['d_root']));
	}
	
	function assertdir($path, $mode = 0777) {
		if (!file_exists($path) || !is_dir($path))
			return $this->mkdir($path, $mode);
		return true;
	}
	
	function mkdir($path, $mode = 0777) {
		if (!@mkdir($path, $mode)) {
			if ($this->_critical_mode) {
				$msg = 'Failed to create directory';
				global $my;
				if ($my->gid==5)
					$msg.=sprintf(' %s', '<small>'.$this->_fix_path($path).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return true;
	}
	
	// warning: $source must exist before calling!
	function copy($source, $dest) {
		if (!@copy($source, $dest)) {
			if ($this->_critical_mode) {
				$msg = 'Failed to copy file';
				global $my;
				if ($my->gid==5)
					$msg.=sprintf(' %s to %s', '<small>'.$this->_fix_path($source).'</small>', '<small>'.$this->_fix_path($dest).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return true;
	}
	
	function _raw_move($source, $dest) {
		// first try a copy/delete approach
		if (!@copy($source, $dest)) {
			if ($this->_critical_mode) {
				$msg = 'Failed to move file';
				global $my;
				if ($my->gid==5)
					$msg.=sprintf(' %s to %s', '<small>'.$this->_fix_path($source).'</small>', '<small>'.$this->_fix_path($dest).'</small>');
				$this->_failure($msg);
			}
			return false;
		}
		return $this->remove($source);
	}
	
	function move($source, $dest) {
		if (!@rename($source, $dest))
			// first try a copy/delete approach
			return $this->_raw_move($source, $dest);
		return true;
	}
	
	function remove($path) {
		if (is_file($path)) {
			if (!@unlink($path)) {
				if ($this->_critical_mode) {
					$msg = 'Failed to remove file';
					global $my;
					if ($my->gid==5)
						$msg.=sprintf(' %s', '<small>'.$this->_fix_path($path).'</small>');
					$this->_failure($msg);
				}
				return false;
			}
			return true;
		}
		return $this->deldir($path);
	}
	
	function rmdir($path) {
		if (!@rmdir($path)) {
			if (!$this->_critical_mode)
				return false;
			$msg = 'Failed to remove directory';
			global $my;
			if ($my->gid==5)
				$msg.=sprintf(' %s', '<small>'.$this->_fix_path($path).'</small>');
			
			if (count(read_dir($path)))
				$msg.=' '.'because it contains files, use ::deldir() instead';
			$this->_failure($msg);
		}
		return false;
	}

	// copies all files and folders from $src to $to
	function xcopy($src, $to) {
		$done = $this->fcopy($src, $to);
		$folders = read_dir($src, 'dir');
		foreach ($folders as $dir) {
			if (!is_dir($to.$dir))
				mkdir($to.$dir);
			$done += xcopy($src.$dir.'/', $to.$dir.'/');
		}
		return $done;
	}
	
	// copies only folders and files in subfolders
	function tcopy($src, $to) {
		$done = 0;
		$folders = read_dir($src, 'dir');
		foreach ($folders as $dir) {
			$this->assertdir($to.$dir);
			$done += $this->fcopy($src.$dir.'/', $to.$dir.'/');
		}
		return $done;
	}

	// copies only files
	function fcopy($src, $to) {
		$done = 0;
		$files = read_dir($src, 'file');
		foreach ($files as $file) {
			if ($this->copy($src.$file, $to.$file)!==FALSE)
				$done++;
		}
		return $done;
	}

	// recursively delete a directory
	// must use trailing slash
	function purge($dir) {
	  $handle = opendir($dir);
	  $noerror = true;
	  while (false !== ($file = readdir($handle))) {
		if ($file{0} != '.') {
		if (is_dir($dir.$file)) {
			$this->purge ($dir.$file."/");
			if(!$this->rmdir($dir.$file))
				$noerror = false;
		}
		else if(!$this->unlink($dir.$file))
			$noerror = false;
		}
	  }
	  closedir($handle);
	  return $noerror;
	}

	// recursively delete a directory and finally remove it too
	// must use trailing slash
	function deldir($dir)
	{
		if ($this->purge($dir))
			return $this->rmdir($dir);
		else
			return false;
	}

	function chmod_all($path, $mode) {
		$handle = opendir($path);
		$noerror = $this->chmod($path, $mode);
		if (!$noerror) return false;
		while (false !== ($file = readdir($handle))) {
			if ($file{0} == '.') continue;
			if (is_dir($path.$file)) {
				$noerror = $this->chmod_all ($path.$file."/", $mode);
				if (!$noerror) break;
			} else {
				$noerror = $this->chmod($path.$file, $mode);
				if (!$noerror) break;
			}
		}
		closedir($handle);
		return $noerror;
	}

	
	// deprecated methods
	function rename($src, $dest) { return $this->move($src, $dest); }
	function unlink($file) { return $this->remove($file); }
	function delete($file) { return $this->remove($file); }

}

?>